@@ -15,15 +15,19 @@ use POE::Component::IRC::Common qw(:ALL);
 use POE::Component::IRC::Plugin qw(:ALL);
 use base qw(POE::Component::IRC);
 use vars qw($VERSION);
+use Data::Dumper;
 
-$VERSION = '2.2';
+$VERSION = '2.41';
 
 # Event handlers for tracking the STATE. $self->{STATE} is used as our namespace.
 # u_irc() is used to create unique keys.
 
 # Make sure we have a clean STATE when we first join the network and if we inadvertently get disconnected
 sub S_001 {
-  delete $_[0]->{STATE};
+  my $self = shift;
+  delete $self->{STATE};
+  $self->{STATE}->{usermode} = '';
+  $self->yield( 'mode', $self->{RealNick} );
   return PCI_EAT_NONE;
 }
 
@@ -216,6 +220,14 @@ sub S_chan_mode {
   return PCI_EAT_NONE;
 }
 
+sub S_221 {
+  my ($self,$irc) = splice @_, 0, 2;
+  my $mode = ${ $_[0] };
+  $mode =~ s/^\+//;
+  ($self->{STATE}->{usermode} = $mode ) =~ s/^\+//;
+  return PCI_EAT_NONE;
+}
+
 # Channel MODE
 sub S_mode {
   my ($self,$irc) = splice @_, 0, 2;
@@ -308,6 +320,20 @@ sub S_mode {
         delete $self->{STATE}->{Chans}->{ $uchan }->{Mode};
      }
   }
+  else {
+     my $parsed_mode = parse_mode_line( @modes );
+     while ( my $mode = shift ( @{ $parsed_mode->{modes} } ) ) {
+        $self->_send_event( 'irc_user_mode', $who, $channel, $mode );
+	if ( my ($flag) = $mode =~ /^\+(.)/ ) {
+		$self->{STATE}->{usermode} .= $flag unless $self->{STATE}->{usermode} =~ /$flag/;
+		next;
+	}
+        if ( my ($flag) = $mode =~ /^-(.)/ ) {
+		$self->{STATE}->{usermode} =~ s/$flag// if $self->{STATE}->{usermode} =~ /$flag/;
+		next;
+	}
+     }
+  }
   return PCI_EAT_NONE;
 }
 
@@ -527,6 +553,21 @@ sub S_333 {
 # Internal methods begin with '_'
 #
 
+sub umode {
+  my $self = shift;
+  return $self->{STATE}->{usermode};
+}
+
+sub is_user_mode_set {
+  my $self = shift;
+  my $mode = ( split //, $_[0] )[0] || return 0;
+  $mode =~ s/[^A-Za-z]//g;
+  return unless $mode;
+  return 1 if $self->{STATE}->{usermode} =~ /$mode/;
+  return 0;
+}
+
+
 sub _channel_sync {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
@@ -546,26 +587,31 @@ sub _channel_sync {
 
 sub _nick_exists {
   my $self = shift;
+  my $nick = shift || return;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $nick = u_irc $_[0], $mapping || return 0;
+  $nick = u_irc $nick, $mapping;
   return 1 if defined $self->{STATE}->{Nicks}->{ $nick };
   return 0;
 }
 
 sub _channel_exists {
   my $self = shift;
+  my $channel = shift || return;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc $_[0], $mapping || return 0;
+  $channel = u_irc $channel, $mapping;
   return 1 if defined $self->{STATE}->{Chans}->{ $channel };
   return 0;
 }
 
 sub _nick_has_channel_mode {
   my $self = shift;
+  my $channel = shift || return;
+  my $nick = shift || return;
+  my $flag = shift || return;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc $_[0], $mapping || return 0;
-  my $nick = u_irc $_[1], $mapping || return 0;
-  my $flag = ( split //, $_[2] )[0] || return 0;
+  $channel = u_irc $channel, $mapping;
+  $nick = u_irc $nick, $mapping;
+  $flag = ( split //, $flag )[0];
 
   return 0 unless $self->is_channel_member($channel,$nick);
   return 1 if $self->{STATE}->{Nicks}->{ $nick }->{CHANS}->{ $channel } =~ /$flag/;
@@ -600,13 +646,9 @@ sub nick_info {
   return unless $self->_nick_exists($nick);
 
   my $record = $self->{STATE}->{Nicks}->{ $nick };
-
   my %result = %{ $record };
-
   $result{Userhost} = $result{User} . '@' . $result{Host};
-
   delete $result{'CHANS'};
-
   return \%result;
 }
 
@@ -968,6 +1010,14 @@ All of the L<POE::Component::IRC> methods are supported, plus the following:
 
 =over
 
+=item umode
+
+Takes no parameters. Returns the current user mode set for the bot.
+
+=item is_user_mode_set
+
+Expects single user mode flag [A-Za-z]. Returns 1 if that user mode is set, 0 otherwise.
+
 =item channels
 
 Takes no parameters. Returns a hashref, keyed on channel name and whether the bot is operator, halfop or 
@@ -1118,6 +1168,10 @@ ARG0 is the user's nickname and ARG1 the channel they have joined.
 This is almost identical to irc_mode, except that it's sent once for each individual mode with it's respective
 argument if it has one (ie. the banmask if it's +b or -b). However, this event is only sent for channel modes.
 
+=item irc_user_mode
+
+This is almost identical to irc_mode, except it is sent for each individual umode that is being set.
+
 =back
 
 The following two 'irc_*' events are the same as their L<POE::Component::IRC> counterparts,
